
%Belyk, September 2017
%The basic idea of this script is to load predefined processing jobs built
%in the GUI with one sample scan in mind, and it automate replacing some of
%its settings (mostly filepaths) to process the remaining scans in the same
%way. This script depends on being able to load the structures (realign,
%coregister, smooth*, segment, and normalize. *smooth is not implemented
%here.

%data should be in 4D.nii format and organized such that each participant
%has a folder within which each scan has a folder.

%anats should already have an origin near the anterior commissure
%use display function in gui to do this

clear all
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%STUFF THAT NEEDS SETTING%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%list of subject folder names
subjects = { 'c1', 'c2','c3b', 'c4', 'c5', 'c10','c6','c11', 'c12', 'c13', 'c14', 'c15'};

%list of run names, should also be name of run folders
runs = {'bold1', 'bold2','bold3', 'bold4'};
no_runs = length(runs); %count 'em

%runs to skip, missing data
skip_runs=["c10bold1"];

%number of volumes per functional run
n_volumes = 256;

%where the data is
basepath = '/Users/Belykm/Documents/MATLAB/Phonotopy_processing/';

%%%SOME SETTINGS%%%
%some settings
spm('defaults','fmri');
spm_jobman('initcfg');

%load spm template jobs
load('realign.mat')
load('coregister.mat')
load('segment.mat')
load('normalize.mat')

%%%Image preprocessing %%%

tic %a time thingy
for s = subjects %loop through subjects
    %steps to do once per subject go here

    disp('%%%%%%%%%%%%%%%%%%%%%%%%')
    disp(strcat("Start Subject: ", s))
    disp('%%%%%%%%%%%%%%%%%%%%%%%%')

    %build path to ANAT folder
    anatpath = fullfile(basepath, s,'anat');


    %%%%%%%%%%%%%
    %%%Segment%%%
    %%%%%%%%%%%%%
    %setup T1 segmentation job
    segment{1}.spm.spatial.preproc.channel.vols = fullfile(anatpath,'anat.nii');%T1

    %execute T1 segmentation job
    spm_jobman('run',segment)
    disp(strcat("Done Segmenting T1: ", s))

    %%%%%%%%%%%%%%%%%%
    %%%Normalize_T1%%%
    %%%%%%%%%%%%%%%%%%
    %setup normalization job
    normalize{1}.spm.spatial.normalise.write.subj.def = fullfile(anatpath,'y_anat.nii');%deformation field
    normalize{1}.spm.spatial.normalise.write.subj.resample = fullfile(anatpath,'anat.nii');

    %execute normalization job
    spm_jobman('run',normalize)
    load('normalize.mat')%reset before ahead of bold normalization
    disp(strcat("Done Normalizing T1: ", s))

    %processing BOLD images
    for r = runs %loop through runs in subjects
        %skip missing runs
        if ismember(strcat(s,r), skip_runs)%should this run be skipped
            disp(strcat("%%% Skipping:", s," ",r,"","%%%"))
        else

            %steps to do for all runs go here
             tic %start timer

             %print to console for keeping track
             disp('%%%%%%%%%%%%%%%%%%%%%%%%')
             disp(strcat("Start Preprocessing: ", s,'/', r))
             disp('%%%%%%%%%%%%%%%%%%%%%%%%')

            %build path to BOLD folders
            boldpath = fullfile(basepath, s,r);

            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %%%REALIGN/MOVEMENT CORRECTION%%%
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            %setup realign job
            for v = 1:n_volumes %loop through volumes
              realign{1}.spm.spatial.realign.estimate.data{1}(v) = strcat(boldpath,'/',r,'.nii,',num2str(v));
            end

            %execute realign job
            spm_jobman('run',realign)
            disp(strcat("Done Realigning: ", s,'/', r))

            %%%%%%%%%%%%%%%%%%%%
            %%%COREGISTRATION%%%
            %%%%%%%%%%%%%%%%%%%%
            %setup coregistration job
            coregister{1}.spm.spatial.coreg.estimate.source = fullfile(anatpath,'anat.nii'); %T1 for coregistering to
            coregister{1}.spm.spatial.coreg.estimate.ref = strcat(boldpath,'/',r,'.nii,1'); %first volume or realigned images

            for v = 2:n_volumes %loop through remaining volumes
                coregister{1}.spm.spatial.coreg.estimate.other{v-1} = strcat(boldpath,'/',r,'.nii,',num2str(v)); %remaining volumes
            end

            %execute coregistration job
            spm_jobman('run',coregister)
            disp(strcat("Done Coregistering: ", s,'/', r))


            %%%PUT SPATIAL SMOOTHING HERE%%%
            %%%was not needed by project for which this script was developed


            %%%%%%%%%%%%%%%%%%%%
            %%%Normalize_Bold%%%
            %%%%%%%%%%%%%%%%%%%%
            %setup normalization job
            normalize{1}.spm.spatial.normalise.write.subj.def = fullfile(anatpath,'y_anat.nii');%deformation field
            for v = 1:n_volumes %loop through volumes
                normalize{1}.spm.spatial.normalise.write.subj.resample(v) = strcat(boldpath,'/',r,'.nii,',num2str(v)); %volumes to normalize
            end

        %execute normalization job
        spm_jobman('run',normalize)
        disp(strcat("Done Normalizing: ", s,'/', r))

        end %finish else
    end %finish run loop
    toc %time for subject
end %finish subject loop

            disp('%%%')
            disp('Fin') %message to console
            disp('%%%')
